using UnityEngine;
using System.Collections.Generic;

public class HexMG : QuizBase 
{
	// Game parameters
	private List<int> hexValues;
	private int targetValue, currentValue;
	private bool displayCurrent;
	
	private int displayTimer;
	
	// Keep track of left and right clicks.
	bool leftClick;
	
	
	// Use this for initialization
	void Start () 
	{
		
	}
	
	protected override void initMinigame()
	{
		int maxTargetValue = 0;
		int numHexValues = 0;
		
		// Apply challenge rating.
		switch(challengeRating)
		{
		case 1:
			maxTargetValue = 255;
			numHexValues = 2;
			displayCurrent = true;
			break;
		case 2:
			maxTargetValue = 4095;
			numHexValues = 3;
			displayCurrent = true;
			break;
		case 3:
			maxTargetValue = 4095;
			numHexValues = 3;
			displayCurrent = false;
			break;
		case 4:
			maxTargetValue = 65535;
			numHexValues = 4;
			displayCurrent = true;
			break;
		case 5:
			maxTargetValue = 65535;
			numHexValues = 4;
			displayCurrent = false;
			break;
		default:
			break;
		}
		
		// Establish current and target values.
		targetValue = Random.Range(1, maxTargetValue);
		currentValue = 0;
		
		// Set up the binary table.
		hexValues = new List<int>();
		for (int i = 0; i < numHexValues; i++)
		{
			hexValues.Add(0);	
		}
		
		// Set up display timer.
		displayTimer = 0;
	}
	
	// Update is called once per frame
	void Update () 
	{
		if (guiActive)
		{
			currentValue = 0;
			for (int i = 0; i < hexValues.Count; i++)
			{
				currentValue += (int)Mathf.Pow(16, i)*hexValues[i];			
			}
			
			if (Input.GetMouseButtonDown(0))
				leftClick = true;
			if (Input.GetMouseButtonDown(1))
				leftClick = false;
			
			if (displayTimer > 0)
				displayTimer--;
		}
	}
	
	void OnGUI()
	{
		if (guiActive)
		{
			// Apply GUISkin
			GUI.skin = skin;
				
			// Background window.
			GUI.Box(new Rect(Screen.width*0.15f, Screen.height*0.2f, Screen.width*0.7f, Screen.height*0.6f), "");
			
			// Help button
			if (GUI.Button(new Rect(Screen.width*0.18f, Screen.height*0.68f, Screen.width*0.05f, Screen.width*0.05f), "?"))
			{
				help = !help;
			}
			
			if (help)
			{
				// Help screen.
				GUI.Label(new Rect(Screen.width*0.2f, Screen.height*0.2f, Screen.width*0.6f, Screen.height*0.6f),
					"Hexadecimal conversion: \n\n" +
					"Convert the number shown to Hexadecimal: \n" +
					"Hexadecimal is a base-16 numeral system; each digit " +
					"represents a power of 16, instead of 10.");
			}
			else
			{
				// Puzzle screen.
				
				// Left number out: target value.
				GUI.Box(new Rect(Screen.width*0.25f, Screen.height*0.25f, Screen.width*0.2f, Screen.height*0.15f), "", "InnerBox");
				GUI.Label(new Rect(Screen.width*0.26f, Screen.height*0.26f, Screen.width*0.18f, 
					Screen.height*0.13f), targetValue.ToString());
				
				// Right number out: current value.
				GUI.Box(new Rect(Screen.width*0.55f, Screen.height*0.25f, Screen.width*0.2f, Screen.height*0.15f), "", "InnerBox");
				if (displayCurrent || displayTimer > 0)
				{
					GUI.Label(new Rect(Screen.width*0.56f, Screen.height*0.26f, Screen.width*0.18f, 
						Screen.height*0.13f), currentValue.ToString());
				}
				
				// Binary switches.
				GUILayout.BeginArea(new Rect(Screen.width*0.2f, Screen.height*0.5f, Screen.width*0.6f, Screen.height*0.1f));
				GUILayout.BeginHorizontal();
				for (int i = hexValues.Count-1; i >= 0; i--)
				{
					GUILayout.FlexibleSpace();
					if (GUILayout.Button(""))
					{
						if (leftClick)
						{
							hexValues[i]++;
							if (hexValues[i] > 15)
								hexValues[i] = 0;
						}
						else
						{
							hexValues[i]--;
							if (hexValues[i] < 0)
								hexValues[i] = 15;
						}
					}
					GUILayout.FlexibleSpace();
				}
				GUILayout.EndHorizontal();
				GUILayout.EndArea();
				
				GUILayout.BeginArea(new Rect(Screen.width*0.2f, Screen.height*0.51f, Screen.width*0.6f, Screen.height*0.1f));
				GUILayout.BeginHorizontal();
				for (int i = 0; i < hexValues.Count; i++)
				{
					GUILayout.FlexibleSpace();
					GUILayout.Label("", hexValues[i].ToString(), GUILayout.Width(48f), GUILayout.Height(48f));
					GUILayout.FlexibleSpace();
				}
				GUILayout.EndHorizontal();
				GUILayout.EndArea();
				
				// Confirm button
				if (GUI.Button(new Rect(Screen.width*0.78f, Screen.height*0.68f, Screen.width*0.05f, Screen.width*0.05f), "OK"))
				{
					if (currentValue == targetValue)
					{
						success = true;
						guiActive = false;
						
						//Debug.Log("If I make it, I become it: "+success); 
						
					 	GameObject player = GameObject.Find("ZigguratPlayer"); 
						//Debug.Log("I am: "+player.transform.position); 
						
						// tLefty
						if (player.transform.position.x < 0 && player.transform.position.z > 0)
						{
							GameObject visible = GameObject.Find("tLefty"); 
							visible.GetComponent<wallVisibilitytLefty>().visibilityOn = true;
						}
						
						// tRighty
						if (player.transform.position.x > 0 && player.transform.position.z > 0)
						{
							GameObject visible = GameObject.Find("tRighty"); 
							visible.GetComponent<wallVisibilitytRighty>().visibilityOn = true;
						}
						
						// bLefty
						if (player.transform.position.x < 0 && player.transform.position.z < 0)
						{
							GameObject visible = GameObject.Find("bLefty"); 
							visible.GetComponent<wallVisibilitybLefty>().visibilityOn = true;
						}
						
						
						if (player.transform.position.x > 0 && player.transform.position.z < 0)
						{
							GameObject visible = GameObject.Find("bRighty"); 
							visible.GetComponent<wallVisibilitybRighty>().visibilityOn = true;
						}
					}
					
					if (!displayCurrent)
						displayTimer = 100;
				}
				// Close button
				if (GUI.Button(new Rect(Screen.width*0.70f, Screen.height*0.68f, Screen.width*0.05f, Screen.width*0.05f), "X"))
				{
					guiActive = false;
				}
			}
		}
	}
}

