using UnityEngine;
using System.Collections.Generic;

public class MastermindMG : QuizBase 
{
	// Game parameters
	private int numVariables, numGuesses, numGuessesTaken, numValues;
	private List<int> correctVariables, variables;
	private List<string> feedback;
	private List<List<int>> guesses;
	private List<bool> usedValues;
	
	private string[] colors = {"Red", "Green", "Blue", "Yellow", "Cyan", "Majenta"};
	
	// Use this for initialization
	void Start () 
	{
		
	}
	
	protected override void initMinigame()
	{
		// Apply challenge rating.
		numGuessesTaken = 0;
		switch(challengeRating)
		{
		case 1:
			numVariables = 3;
			numValues = 3;
			numGuesses = 10;
			break;
		case 2:
			numVariables = 4;
			numValues = 3;
			numGuesses = 9;
			break;
		case 3:
			numVariables = 4;
			numValues = 4;
			numGuesses = 8;
			break;
		case 4:
			numVariables = 5;
			numValues = 4;
			numGuesses = 7;
			break;
		case 5:
			numVariables = 5;
			numValues = 5;
			numGuesses = 6;
			break;
		default:
			break;
		}
		
		// Tracks which values are actually used in the puzzle.
		usedValues = new List<bool>();
		for (int i = 0; i < numValues; i++)
			usedValues.Add(false);
		
		// Populate variable list.
		variables = new List<int>();
		correctVariables = new List<int>();
		for (int i = 0; i < numVariables; i++)
		{
			int rand = Random.Range(0, numValues);
			correctVariables.Add(rand);
			variables.Add(0);
			usedValues[rand] = true;
		}
		
		// Prepare feedback list.
		feedback = new List<string>();
		guesses = new List<List<int>>();
		for (int i = 0; i < numGuesses; i++)
		{
			feedback.Add("");
		}
		
	}
	
	// Update is called once per frame
	void Update () 
	{
		if (guiActive)
		{
			if (numGuessesTaken == numGuesses)
			{
				// Reset puzzle.
				initMinigame();
			}
		}
	}
	
	void OnGUI()
	{
		if (guiActive)
		{
			// Apply GUISkin
			GUI.skin = skin;
				
			// Background window.
			GUI.Box(new Rect(Screen.width*0.15f, Screen.height*0.05f, Screen.width*0.7f, Screen.height*0.9f), "");
			
			// Help button
			if (GUI.Button(new Rect(Screen.width*0.18f, Screen.height*0.8f, Screen.width*0.05f, Screen.width*0.05f), "?"))
			{
				help = !help;
			}
			
			if (help)
			{
				// Help screen.
				GUI.Label(new Rect(Screen.width*0.2f, Screen.height*0.2f, Screen.width*0.6f, Screen.height*0.6f),
					"Mastermind: \n\n" +
					"Try and guess the right color combination. \n" +
					"For each incorrect guess, you will be told the number of colors you: \n" +
					"1. Guessed correct, and \n" +
					"2. Guessed correct and in the correct location.");
			}
			else
			{
				// Puzzle screen.
				Vector2 pos;
				
				// Value Buttons
				GUILayout.BeginArea(new Rect(Screen.width*0.2f, Screen.height*0.1f, Screen.width*0.6f, Screen.height*0.3f));
				GUILayout.BeginHorizontal();
				for (int i = 0; i < numVariables; i++)
				{
					GUILayout.FlexibleSpace();
					if (GUILayout.Button(""))
					{
						variables[i]++;
						if (variables[i] >= numValues)
						{
							variables[i] = 0;
						}
					}
					GUILayout.FlexibleSpace();
				}
				GUILayout.EndHorizontal();
				GUILayout.EndArea();
				
				GUILayout.BeginArea(new Rect(Screen.width*0.2f, Screen.height*0.1f, Screen.width*0.6f, Screen.height*0.3f));
				GUILayout.BeginHorizontal();
				for (int i = 0; i < numVariables; i++)
				{
					GUILayout.FlexibleSpace();
					GUILayout.Label("", colors[variables[i]], GUILayout.Width(64f), GUILayout.Height(64f));
					GUILayout.FlexibleSpace();
				}
				GUILayout.EndHorizontal();
				GUILayout.EndArea();
				//GUI.DrawTexture(new Rect(Screen.width*0.2f, Screen.height*0.2f, Screen.width*0.1f, Screen.width*0.1f), red);
				
				// Guesses List
				pos = new Vector2(Screen.width*0.3f, Screen.height*0.3f);
				for (int i = 0; i < numGuesses; i++)
				{
					if (i < guesses.Count)
					{
						string vals = "";
						for (int j = 0; j < guesses[i].Count; j++)
						{
							vals += guesses[i][j].ToString();
						}
						
						// Display the string of values.
						GUI.Label(new Rect(pos.x, pos.y + (Screen.height*0.05f*i), Screen.width*0.15f, Screen.height*0.15f),
							vals, "MastermindLabel");
					}
					else
					{
						// Display the string of values.
						GUI.Label(new Rect(pos.x, pos.y + (Screen.height*0.05f*i), Screen.width*0.15f, Screen.height*0.15f),
							"???", "MastermindLabel");
					}
				}
				
				// Feedback list
				pos.x = Screen.width*0.55f;
				for (int i = 0; i < feedback.Count; i++)
				{
					GUI.Label(new Rect(pos.x, pos.y + (Screen.height*0.05f*i) - (Screen.height*0.05f*numGuesses), 
						Screen.width*0.15f, Screen.height*0.15f), feedback[i], "MastermindLabel");
				}
				
				// Confirm button
				if (GUI.Button(new Rect(Screen.width*0.78f, Screen.height*0.8f, Screen.width*0.05f, Screen.width*0.05f), "OK"))
				{
					bool result = true;
					for (int i = 0; i < variables.Count; i++)
					{
						if (variables[i] != correctVariables[i])
							result = false;
					}
					
					if (result)
					{
						success = true;
						guiActive = false;
						
						//Debug.Log("If I make it, I become it: "+success); 
						
					 	GameObject player = GameObject.Find("ZigguratPlayer"); 
						//Debug.Log("I am: "+player.transform.position); 
						
						// tLefty
						if (player.transform.position.x < 0 && player.transform.position.z > 0)
						{
							GameObject visible = GameObject.Find("tLefty"); 
							visible.GetComponent<wallVisibilitytLefty>().visibilityOn = true;
						}
						
						// tRighty
						if (player.transform.position.x > 0 && player.transform.position.z > 0)
						{
							GameObject visible = GameObject.Find("tRighty"); 
							visible.GetComponent<wallVisibilitytRighty>().visibilityOn = true;
						}
						
						// bLefty
						if (player.transform.position.x < 0 && player.transform.position.z < 0)
						{
							GameObject visible = GameObject.Find("bLefty"); 
							visible.GetComponent<wallVisibilitybLefty>().visibilityOn = true;
						}
						
						
						if (player.transform.position.x > 0 && player.transform.position.z < 0)
						{
							GameObject visible = GameObject.Find("bRighty"); 
							visible.GetComponent<wallVisibilitybRighty>().visibilityOn = true;
						}
					}
					else
					{
						int numCorrect = 0;
						int numCorrectSpot = 0;
						
						for (int i = 0; i < variables.Count; i++)
						{
							// Judge correct value.
							if (usedValues[variables[i]])
							{
								numCorrect++;
							}
							
							// Judge correct spot.
							if (correctVariables[i] == variables[i])
							{
								numCorrectSpot++;
							}
						}
						
						List<int> guess = new List<int>();
						for (int i = 0; i < variables.Count; i++)
						{
							guess.Add(variables[i]);	
						}
						
						guesses.Add(guess);
						feedback.Add(numCorrect.ToString() + " correct : " + numCorrectSpot.ToString() + " in the correct location.");
						
						numGuessesTaken++;
					}
				}
				// Close button
				if (GUI.Button(new Rect(Screen.width*0.70f, Screen.height*0.8f, Screen.width*0.05f, Screen.width*0.05f), "X"))
				{
					guiActive = false;
				}
			}
		}
	}
}
