using UnityEngine;
using System.Collections.Generic;

public class SudokuMG : QuizBase 
{
	// 2D list containing sudoku values.
	private List<List<int>> gridNodes;
	private List<List<bool>> modifiable;
	
	private int gridSize, numTrue, numFalse;
	
	// Use this for initialization
	void Start () 
	{
		
	}
	
	protected override void initMinigame()
	{
		// Establish grid size based on challenge rating
		switch (challengeRating)
		{
		case 1:
			gridSize = 2;
			break;
		case 2:
			gridSize = 4;
			break;
		case 3:
			gridSize = 6;
			break;
		case 4:
			gridSize = 8;
			break;
		case 5:
			gridSize = 10;
			break;
		default:
			gridSize = 10;
			break;
		}
		
		// Create and initialize grid node 2D list.
		gridNodes = new List<List<int>>();
		modifiable = new List<List<bool>>();
		for (int i = 0; i < gridSize; i++)
		{
			gridNodes.Add(new List<int>());
			modifiable.Add(new List<bool>());
			
			for (int j = 0; j < gridSize; j++)
			{
				gridNodes[i].Add(-1);
				modifiable[i].Add(false);
			}
		}
		
		// Populate grid randomly.
		PopulateGrid();
		/*
		for (int i = 0; i < gridSize; i++)
		{
			for (int j = 0; j < gridSize; j++)
			{
				gridNodes[i][j] = Random.Range(0, 2);
			}
		}
		
		// Refine the grid until we have a proper one.
		bool result = false;
		int iter = 0;
		while (!result && iter < 10)
		{
			result = RefineGrid();
			iter++;
		}
		*/
		// Set around half of the nodes' values to -1 (representing blank space).
		for (int i = 0; i < (gridSize*gridSize/2) + (challengeRating*2) - 1; i++)
		{
			int rr = Random.Range(0, gridSize);
			int rc = Random.Range(0, gridSize);
			
			gridNodes[rr][rc] = -1;
			modifiable[rr][rc] = true;
		}
	}
	
	// Update is called once per frame
	void Update () 
	{

	}
	
	void OnGUI()
	{
		if (guiActive)
		{
			// Apply GUISkin
			GUI.skin = skin;
				
			// Background window.
			GUI.Box(new Rect(Screen.width*0.15f, Screen.height*0.05f, Screen.width*0.7f, Screen.height*0.9f), "");
			
			// Help button
			if (GUI.Button(new Rect(Screen.width*0.18f, Screen.height*0.8f, Screen.width*0.05f, Screen.width*0.05f), "?"))
			{
				help = !help;
			}
			
			if (help)
			{
				// Help screen.
				GUI.Label(new Rect(Screen.width*0.2f, Screen.height*0.2f, Screen.width*0.6f, Screen.height*0.6f),
					"Binary Sudoku: \n\n" +
					"Fill the grid with one symbol or another. You may not: \n" +
					"1. Have three of the same symbol in a row, or \n" +
					"2. Have more than half of any one row or column contain the same symbol.");
			}
			else
			{
				// Puzzle screen.
				Vector2 pos = new Vector2(Screen.width*0.425f - (23.0f*((float)challengeRating-1)),
					Screen.height*0.30f - (20.0f*((float)challengeRating-1)));
				string style = "";
				
				for (int i = 0; i < gridSize; i++)
				{
					for (int j = 0; j < gridSize; j++)
					{
						switch(gridNodes[i][j])
						{
						case 0:
							style = "ZeroButton";
							break;
						case 1:
							style = "OneButton";
							break;
						default:
							style = "BlankButton";
							break;
						}
						
						if (modifiable[i][j])
						{
							if (GUI.Button(new Rect(pos.x + (i*Screen.width*0.10f/(1 + ((float)challengeRating/3))), 
									pos.y + (j*Screen.width*0.10f/(1 + ((float)challengeRating/3))), 
									Screen.width*0.10f/(1 + ((float)challengeRating/3)), 
									Screen.width*0.10f/(1 + ((float)challengeRating/3))), "", style))
							{
								gridNodes[i][j]++;
								if (gridNodes[i][j] > 1)
									gridNodes[i][j] = -1;
							}
						}
						else
						{
							GUI.Label(new Rect(pos.x + (i*Screen.width*0.10f/(1 + ((float)challengeRating/3))), 
									pos.y + (j*Screen.width*0.10f/(1 + ((float)challengeRating/3))), 
									Screen.width*0.10f/(1 + ((float)challengeRating/3)), 
									Screen.width*0.10f/(1 + ((float)challengeRating/3))), "", style);
						}
					}
				}
				
				// Confirm button
				if (GUI.Button(new Rect(Screen.width*0.78f, Screen.height*0.8f, Screen.width*0.05f, Screen.width*0.05f), "OK"))
				{
					if (CheckGrid())
					{
						success = true;
						guiActive = false;
						
						//Debug.Log("If I make it, I become it: "+success); 
						
					 	GameObject player = GameObject.Find("ZigguratPlayer"); 
						//Debug.Log("I am: "+player.transform.position); 
						
						// tLefty
						if (player.transform.position.x < 0 && player.transform.position.z > 0)
						{
							GameObject visible = GameObject.Find("tLefty"); 
							visible.GetComponent<wallVisibilitytLefty>().visibilityOn = true;
						}
						
						// tRighty
						if (player.transform.position.x > 0 && player.transform.position.z > 0)
						{
							GameObject visible = GameObject.Find("tRighty"); 
							visible.GetComponent<wallVisibilitytRighty>().visibilityOn = true;
						}
						
						// bLefty
						if (player.transform.position.x < 0 && player.transform.position.z < 0)
						{
							GameObject visible = GameObject.Find("bLefty"); 
							visible.GetComponent<wallVisibilitybLefty>().visibilityOn = true;
						}
						
						
						if (player.transform.position.x > 0 && player.transform.position.z < 0)
						{
							GameObject visible = GameObject.Find("bRighty"); 
							visible.GetComponent<wallVisibilitybRighty>().visibilityOn = true;
						}
					}
				}
				// Close button
				if (GUI.Button(new Rect(Screen.width*0.70f, Screen.height*0.8f, Screen.width*0.05f, Screen.width*0.05f), "X"))
				{
					guiActive = false;
				}
				
				// Reset button
				if (GUI.Button(new Rect(Screen.width*0.25f, Screen.height*0.8f, Screen.width*0.05f, Screen.width*0.05f), "R"))
				{
					initMinigame();
				}
			}
		}
	}
	
	void PopulateGrid()
	{
		List<List<int>> tempGrid = new List<List<int>>();
		
		for (int i = 0; i < gridSize; i++)
		{
			tempGrid.Add(new List<int>());
		}
		
		// Randomly spawn the grid in 2x2 chunks.
		for (int i = 0; i < gridSize / 2; i++)
		{
			for (int j = 0; j < gridSize / 2; j++)
			{
				int r = Random.Range (0, 2);
				
				if (r == 0)
				{
					tempGrid[i * 2].Add (0); //tempGrid[i * 2][j * 2] = 0;
					tempGrid[i * 2].Add (1); //tempGrid[i * 2][j * 2 + 1] = 1;
					tempGrid[i * 2 + 1].Add (1); //tempGrid[i * 2 + 1][j * 2] = 1;
					tempGrid[i * 2 + 1].Add (0); //tempGrid[i * 2 + 1][j * 2 + 1] = 0;
				}
				else
				{
					tempGrid[i * 2].Add (1); //tempGrid[i * 2][j * 2] = 1;
					tempGrid[i * 2].Add (0); //tempGrid[i * 2][j * 2 + 1] = 0;
					tempGrid[i * 2 + 1].Add (0); //tempGrid[i * 2 + 1][j * 2] = 0;
					tempGrid[i * 2 + 1].Add (1); //tempGrid[i * 2 + 1][j * 2 + 1] = 1;
				}
			}
		}
		
		// Offset the grid by a random amount.
		int offsetX = Random.Range(0, 2);
		int offsetY = Random.Range(0, 2);
		
		// Copy the temporary grid into the main grid.
		for (int i = 0; i < gridSize; i++)
		{
			for (int j = 0; j < gridSize; j++)
			{
				gridNodes[i][j] = tempGrid[(i + offsetY) % gridSize][(j + offsetX) % gridSize];
				Debug.Log ("Space " + i + "," + j + " = " + gridNodes[i][j]);
			}
		}
	}
	
	bool CheckGrid()
	{
		List<int> rows0 = new List<int>();
		List<int> cols0 = new List<int>();
		List<int> rows1 = new List<int>();
		List<int> cols1 = new List<int>();
		
		for (int i = 0; i < gridSize; i++)
		{
			rows0.Add(0);
			cols0.Add(0);
			rows1.Add(0);
			cols1.Add(0);
		}
		
		for (int i = 0; i < gridSize; i++)
		{
			for (int j = 0; j < gridSize; j++)
			{
				switch (gridNodes[i][j])
				{
				case 0:
					rows0[i]++;
					cols0[j]++;
					break;
				case 1:
					rows1[i]++;
					cols1[j]++;
					break;
				default:
					// The puzzle isn't solved until all empty spaces are filled in.
					return false;
				}
				
				// If there are more 0s or 1s in a row or column than half the
				// grid length, we have an incorrect grid.
				if (rows0[i] >  gridSize/2 || rows1[i] > gridSize/2 || 
					cols0[j] > gridSize/2 || cols1[j] > gridSize/2)
				{
					Debug.Log("uneven amount of 0 or 1");
					return false;
				}
				
				// Check the values to the left, right, up and down from the current node.
				// (Don't use the edge nodes, and don't use this in the case of -1 -- a blank space.)
				if (gridNodes[i][j] != -1)
				{
					
					if (i != 0 && i != gridSize - 1)
					{
						if (gridNodes[i][j] == gridNodes[i-1][j] && gridNodes[i][j] == gridNodes[i+1][j])
						{
							Debug.Log("three in a row");
							return false;
						}
					}
					
					if (j != 0 && j != gridSize - 1)
					{
						if (gridNodes[i][j] == gridNodes[i][j-1] && gridNodes[i][j] == gridNodes[i][j+1])
						{
							Debug.Log("three in a row");
							return false;
						}
					}
					
				}
			}
		}
		return true;
	}
	
	bool RefineGrid()
	{
		bool correct = true;
		
		List<int> rows0 = new List<int>();
		List<int> cols0 = new List<int>();
		List<int> rows1 = new List<int>();
		List<int> cols1 = new List<int>();
		
		for (int i = 0; i < gridSize; i++)
		{
			rows0.Add(0);
			cols0.Add(0);
			rows1.Add(0);
			cols1.Add(0);
		}
		
		for (int i = 0; i < gridSize; i++)
		{
			for (int j = 0; j < gridSize; j++)
			{
				// Calculate the totals of all rows and columns.
				switch (gridNodes[i][j])
				{
				case 0:
					rows0[i]++;
					cols0[j]++;
					break;
				case 1:
					rows1[i]++;
					cols1[j]++;
					break;
				default:
					break;
				}
				
				// If there are more 0s or 1s in a row or column than half the
				// grid length, we have an incorrect grid.
				bool result;
				
				if (rows0[i] >  gridSize/2)
				{
					result = false;
					int rand = Random.Range(0, gridSize);
					while (!result)
					{
						if (gridNodes[i][rand] == 0)
						{
							result = true;
							gridNodes[i][rand] = 1;
						}
						else
						{
							rand++;
							if (rand >= gridSize)
								rand = 0;
						}
					}
					correct = false;
				}
				
				if (rows1[i] > gridSize/2)
				{
					result = false;
					int rand = Random.Range(0, gridSize);
					while (!result)
					{
						if (gridNodes[i][rand] == 1)
						{
							result = true;
							gridNodes[i][rand] = 0;
						}
						else
						{
							rand++;
							if (rand >= gridSize)
								rand = 0;
						}
					}
					correct = false;
				}
				
				/*
				if (cols0[i] >  gridSize/2)
				{
					result = false;
					int rand = Random.Range(0, gridSize);
					while (!result)
					{
						if (gridNodes[rand][j] == 0)
						{
							result = true;
							gridNodes[rand][j] = 1;
						}
						else
						{
							rand++;
							if (rand >= gridSize)
								rand = 0;
						}
					}
					correct = false;
				}
				
				if (cols1[i] > gridSize/2)
				{
					result = false;
					int rand = Random.Range(0, gridSize);
					while (!result)
					{
						if (gridNodes[rand][j] == 1)
						{
							result = true;
							gridNodes[rand][j] = 0;
						}
						else
						{
							rand++;
							if (rand >= gridSize)
								rand = 0;
						}
					}
					correct = false;
				}
				*/
				
				// Check the values to the left, right, up and down from the current node.
				// (Don't use the edge nodes, and don't use this in the case of -1 -- a blank space.)
				if (gridNodes[i][j] != -1)
				{
					if (i != 0 && i != gridSize - 1)
					{
						if (gridNodes[i][j] == gridNodes[i-1][j] && gridNodes[i][j] == gridNodes[i+1][j])
						{
							if (gridNodes[i][j] == 0)
								gridNodes[i][j] = 1;
							else if (gridNodes[i][j] == 1)
								gridNodes[i][j] = 0;
							correct = false;
						}
					}
					
					if (j != 0 && j != gridSize - 1)
					{
						if (gridNodes[i][j] == gridNodes[i][j-1] && gridNodes[i][j] == gridNodes[i][j+1])
						{
							if (gridNodes[i][j] == 0)
								gridNodes[i][j] = 1;
							else if (gridNodes[i][j] == 1)
								gridNodes[i][j] = 0;
							correct = false;
						}
					}
					
				}
			}
		}
		
		return correct;
	}
}